@@ -996,7 +996,7 @@ static lookup_t node40_lookup(reiser4_node_t *entity,
 	func = cb_comp_key3;
 #elif defined(ENABLE_LARGE_KEYS)
 	func = cb_comp_key4;
-#elif
+#else
 	func = NULL;
 #endif
 	
