@@ -1,23 +1,57 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    sdext_plug.c -- plugin id stat data extension plugin. */
 
 #include "sdext_plug.h"
 
-reiser4_core_t *sdext_plug_core = NULL;
+reiser4_core_t *sdext_pset_core = NULL;
+
+#ifndef ENABLE_MINIMAL
+static void sdext_plug_info(stat_entity_t *stat) {
+	sdext_plug_t *ext;
+	uint8_t i;
+
+	stat->info.digest = NULL;
+	
+	/* When inserting new extentions, nothing to be done. */
+	if (stat->plug->p.id.id != SDEXT_PSET_ID || !stat) 
+		return;
+	
+	ext = (sdext_plug_t *)stat_body(stat);
+	for (i = 0; i < sdext_plug_get_count(ext); i++) {
+		rid_t mem, id;
+		
+		mem = sdext_plug_get_member(ext, i);
+		id = sdext_plug_get_pid(ext, i);
+
+		if (mem != SDEXT_PSET_ID)
+			continue;
+		
+		stat->info.digest = sdext_pset_core->pset_ops.find(mem, id, 1);
+		if (stat->info.digest == INVAL_PTR)
+			stat->info.digest = NULL;
+		
+		return;
+	}
+}
+#endif
 
 uint32_t sdext_plug_length(stat_entity_t *stat, void *hint) {
 	uint16_t count = 0;
+	uint64_t mask;
+	
+	aal_assert("vpf-1844", stat != NULL || hint != NULL);
 	
 	/* If hint is given, count its pset. */
 	if (hint) {
 		sdhint_plug_t *h = (sdhint_plug_t *)hint;
-		uint16_t i;
-
-		for (i = 0; i < OPSET_STORE_LAST; i++) {
-			if (h->mask & (1 << i))
-				count++;
+		
+		mask = h->plug_mask;
+		
+		while (mask) {
+			if (mask & 1) count++;
+			mask >>= 1;
 		}
 	} else {
 		sdext_plug_t *plug = (sdext_plug_t *)stat_body(stat);
@@ -33,6 +67,7 @@ uint32_t sdext_plug_length(stat_entity_t *stat, void *hint) {
 static errno_t sdext_plug_open(stat_entity_t *stat, void *hint) {
 	sdhint_plug_t *plugh;
 	sdext_plug_t *ext;
+	int is_pset;
 	uint16_t i;
 	
 	aal_assert("vpf-1597", stat != NULL);
@@ -40,30 +75,33 @@ static errno_t sdext_plug_open(stat_entity_t *stat, void *hint) {
 
 	plugh = (sdhint_plug_t *)hint;
 	ext = (sdext_plug_t *)stat_body(stat);
+
+	is_pset = stat->plug->p.id.id == SDEXT_PSET_ID;
 	
-	aal_memset(plugh, 0, sizeof(*plugh));
+	aal_memset(plugh, 0, sizeof(sdhint_plug_t));
 	
 	for (i = 0; i < sdext_plug_get_count(ext); i++) {
 		rid_t mem, id;
-
+		
 		mem = sdext_plug_get_member(ext, i);
 		id = sdext_plug_get_pid(ext, i);
 		
 		/* Check the member id valideness. */
-		if (mem >= OPSET_STORE_LAST)
+		if (mem >= PSET_STORE_LAST)
 			return -EIO;
 		
 		/* Check if we met this member already. */
-		if (plugh->plug[mem])
+		if (plugh->plug_mask & (1 << mem))
 			return -EIO;
-
+		
 		/* Obtain the plugin by the id. */
-		plugh->plug[mem] = sdext_plug_core->pset_ops.find(mem, id);
+		plugh->plug[mem] = sdext_pset_core->pset_ops.find(
+						mem, id, is_pset);
 		
 		if (plugh->plug[mem] == INVAL_PTR) {
 #ifndef ENABLE_MINIMAL
 			aal_error("Node (%llu), item (%u): Failed to find "
-				  "a plugin of the opset member (%u), id "
+				  "a plugin of the pset member (%u), id "
 				  "(%u).", place_blknr(stat->place),
 				  stat->place->pos.item, mem, id);
 			return -EIO;
@@ -71,15 +109,13 @@ static errno_t sdext_plug_open(stat_entity_t *stat, void *hint) {
 			plugh->plug[mem] = NULL;
 #endif
 		}
-#if 0		
-		/* If the plug is valid but cannot be found (not implemented 
-		   yet), set INVAL_PTR into the opset (to let upper levels to 
-		   know that smth is stored here, but there is no such plugin). 
-		 */
-		if (!h->plug[mem])
-			h->plug[mem] = INVAL_PTR;
-#endif
-		plugh->mask |= (1 << mem);
+		
+		/* For those where no plugin is found but the id is correct, 
+		   keep the id in plug set, remember it is a parameter. */
+		if (plugh->plug[mem] == NULL)
+			plugh->plug[mem] = (void *)id;
+		
+		plugh->plug_mask |= (1 << mem);
 	}
 
 	return 0;
@@ -88,8 +124,10 @@ static errno_t sdext_plug_open(stat_entity_t *stat, void *hint) {
 #ifndef ENABLE_MINIMAL
 static errno_t sdext_plug_init(stat_entity_t *stat, void *hint) {
 	sdhint_plug_t *plugh;
+	tree_entity_t *tree;
 	sdext_plug_t *ext;
 	uint16_t count = 0;
+	uint16_t id;
 	rid_t mem;
 	
 	aal_assert("vpf-1600", stat != NULL);
@@ -97,21 +135,25 @@ static errno_t sdext_plug_init(stat_entity_t *stat, void *hint) {
 	
 	plugh = (sdhint_plug_t *)hint;
 	ext = (sdext_plug_t *)stat_body(stat);
+	tree = stat->place->node->tree;
 	
-	for (mem = 0; mem < OPSET_STORE_LAST; mem++) {
+	for (mem = 0; mem < PSET_STORE_LAST; mem++) {
 		/* Find the plugin to be stored. */
-		if (!(plugh->mask & (1 << mem)))
+		if (!(plugh->plug_mask & (1 << mem)))
 			continue;
 
 		sdext_plug_set_member(ext, count, mem);
-		sdext_plug_set_pid(ext, count, plugh->plug[mem] ? 
-				   plugh->plug[mem]->id.id : 0);
 
+		id = (tree->param_mask & (1 << mem)) ? 
+			((uint32_t)plugh->plug[mem]) : 
+			plugh->plug[mem]->id.id;
+		
+		sdext_plug_set_pid(ext, count, id);
 		count++;
 	}
 	
 	sdext_plug_set_count(ext, count);
-
+	
 	return 0;
 }
 
@@ -124,32 +166,44 @@ extern void sdext_plug_print(stat_entity_t *stat,
 
 #endif
 
-static reiser4_sdext_ops_t sdext_plug_ops = {
-	.open	 	= sdext_plug_open,
+reiser4_sdext_plug_t sdext_pset_plug = {
+	.p = {
+		.id    = {SDEXT_PSET_ID, 0, SDEXT_PLUG_TYPE},
+#ifndef ENABLE_MINIMAL
+		.label = "sdext_plugin_set",
+		.desc  = "Plugin Set StatData extension plugin.",
+#endif
+	},
+
 #ifndef ENABLE_MINIMAL
 	.init	 	= sdext_plug_init,
+	.info		= sdext_plug_info,
 	.print   	= sdext_plug_print,
 	.check_struct   = sdext_plug_check_struct,
+#else
+	.info		= NULL,
 #endif
+	.open	 	= sdext_plug_open,
 	.length	 	= sdext_plug_length
 };
 
-static reiser4_plug_t sdext_plug_plug = {
-	.cl    = class_init,
-	.id    = {SDEXT_PLUG_ID, 0, SDEXT_PLUG_TYPE},
+reiser4_sdext_plug_t sdext_hset_plug = {
+	.p = {
+		.id    = {SDEXT_HSET_ID, 0, SDEXT_PLUG_TYPE},
 #ifndef ENABLE_MINIMAL
-	.label = "sdext_plug",
-	.desc  = "Plugin id stat data extension for reiser4. ",
+		.label = "sdext_heir_set",
+		.desc  = "Heir Set StatData extension plugin.",
 #endif
-	.o = {
-		.sdext_ops = &sdext_plug_ops
-	}
+	},
+
+#ifndef ENABLE_MINIMAL
+	.init	 	= sdext_plug_init,
+	.info		= sdext_plug_info,
+	.print   	= sdext_plug_print,
+	.check_struct   = sdext_plug_check_struct,
+#else
+	.info		= NULL,
+#endif
+	.open	 	= sdext_plug_open,
+	.length	 	= sdext_plug_length
 };
-
-static reiser4_plug_t *sdext_plug_start(reiser4_core_t *c) {
-	sdext_plug_core = c;
-	return &sdext_plug_plug;
-}
-
-plug_register(sdext_plug, sdext_plug_start, NULL);
-
